@@ -4,6 +4,248 @@
 # Licence: GPL-3
 ########################### FUNCTIONS FOR TREE DETECTION #######################
 #
+#-------------------------------------------------------------------------------
+#' Tree detection
+#' 
+#' Performs tree detection by applying the functions \code{\link{tree_segmentation}}
+#' and \code{\link{tree_extraction}} to objects of class \code{\link[terra]{SpatRaster-class}},
+#' \code{\link[lidR]{LAS-class}} or \code{\link[lidR]{LAScatalog-class}}
+#' 
+#' @param las An object of class \code{\link[terra]{SpatRaster-class}},
+#' \code{\link[lidR]{LAS-class}} or \code{\link[lidR]{LAScatalog-class}}
+#' @param res numeric. The size of a grid cell in point cloud coordinates units,
+#' used to rasterize the point cloud. In case the \code{las} argument is a \code{SpatRaster}
+#' \code{res} is not used.
+#' @param ROI spatial polygons in sf/sfc format, in the same CRS as argument \code{las}. geometric object that defines the
+#' region where tree detection has to be performed. In case the input is of class
+#' \code{\link[lidR]{LAScatalog-class}}, the chunk buffer set with 
+#' \code{\link[lidR]{engine_options}} is applied to the point cloud to prevent 
+#' border effects, but only treetops lying within the \code{ROI} are returned.
+#' @param normalize boolean. Should the point cloud be normalized before detection
+#' (not applicable if \code{las} argument is a \code{SpatRaster}) ?
+#' @param crown Parameter passed to \code{\link{tree_extraction}}
+#' @param ... Parameters passed to \code{\link{tree_segmentation}}
+#' @examples
+#' # load canopy height model
+#' data(chm_chablais3)
+#' chm_chablais3 <- terra::rast(chm_chablais3)
+#' # create polygon of region of interest
+#' ROI <- sf::st_polygon(list(cbind(
+#'  c(974360, 974360, 974380, 974380, 974360),
+#'  c(6581640, 6581680, 6581680, 6581640, 6581640)
+#' )))
+#' # convert to sfc and set projection
+#' ROI = sf::st_sfc(ROI)
+#' sf::st_crs(ROI) <- terra::crs(chm_chablais3)
+#' #
+#' # tree detection
+#' trees <- tree_detection(chm_chablais3)
+#' # plot results
+#' # canopy height model background
+#' terra::plot(chm_chablais3)
+#' # detected trees
+#' plot(trees["h"], add = TRUE, cex = trees$h/20, col = "black")
+#' #
+#' # tree detection in ROI and minimum tree height set to 10
+#' trees_ROI <- tree_detection(chm_chablais3, ROI = ROI, hmin = 10, crown = TRUE)
+#' # create polygons from WKT field
+#' trees_ROI_crowns <- sf::st_as_sf(sf::st_drop_geometry(trees_ROI), wkt = "crown")
+#' # plot results
+#' # canopy height model background 
+#' terra::plot(chm_chablais3)
+#' # detected trees
+#' plot(trees_ROI["h"], add = TRUE, cex = trees_ROI$h/20, col = "black")
+#' # corresponding crowns
+#' plot(sf::st_geometry(trees_ROI_crowns), add = TRUE, border = "black", col = NA)
+#' # add ROI
+#' plot(ROI, add = TRUE, border = "red", col = NA)
+#' @seealso \code{\link{tree_segmentation}}, \code{\link{tree_extraction}}
+#' @references Monnet, J.-M. 2011. Using airborne laser scanning for mountain 
+#' forests mapping: Support vector regression for stand parameters estimation 
+#' and unsupervised training for treetop detection. Ph.D. thesis. University of 
+#' Grenoble, France. Section 6.2 
+#' \url{https://tel.archives-ouvertes.fr/tel-00652698/document}
+#'
+#' Monnet, J.-M., Mermin, E., Chanussot, J., Berger, F. 2010. Tree top detection 
+#' using local maxima filtering: a parameter sensitivity analysis. Silvilaser 2010, 
+#' the 10th International Conference on LiDAR Applications for Assessing Forest 
+#' Ecosystems, September 14-17, Freiburg, Germany, 9 p. 
+#' \url{https://hal.archives-ouvertes.fr/hal-00523245/document}
+#' @return A sf collection of POINTs with 7 fields: tree id, local maximum stats
+#'  (height, dominance radius), segment stats (surface and volume), coordinates 
+#'  (x and y). In case argument \code{crown} is \code{TRUE}, a \code{crown} field 
+#'  containing the WKT geometry of the 2D crown is also present.
+#' @export
+tree_detection <-
+  function(las,
+           res = 1,
+           ROI = NULL,
+           normalize = FALSE,
+           crown = FALSE,
+           ...
+           # nl_filter = "Closing",
+           # nl_size = 5,
+           # sigma = 0.3,
+           # dmin = 0,
+           # dprop = 0.05,
+           # hmin = 5,
+           # crown_prop = 0.3,
+           # crown_hmin = 2
+           )
+  {
+    if (!is.null(ROI))
+    {
+      # dissolve ROI, keep only geometry
+      ROI <- sf::st_union(sf::st_combine(sf::st_geometry(ROI)))
+    }
+    # if catalog
+    if (lidR::is(las, "LAScatalog")) {
+      # automerge
+      options <- list(automerge = TRUE)
+      # if ROI not NULL
+      if (!is.null(ROI))
+      {
+        if (sf::st_crs(las) != sf::st_crs(ROI))
+          warning("CRS of las and ROI should be the same")
+        # flag as processed only tiles which intersect ROI
+        las$processed <- FALSE
+        dummy <- sf::st_filter(las@data, ROI)
+        las$processed[is.element(las$filename, dummy$filename)] <-
+          TRUE
+        if (sum(las$processed) == 0)
+        {
+          warning("No file intersects the ROI")
+          return(NULL)
+        }
+      }
+      # check buffer size
+      if (lidR::opt_chunk_buffer(las) < 10)
+        warning("For tree segmentation a buffer larger than 10 m is recommended to avoid border effects")
+      # apply function to catalog
+      output <- lidR::catalog_apply(
+        las,
+        tree_detection,
+        res = res,
+        ROI = ROI,
+        normalize = normalize,
+        crown = crown,
+        ...,
+        # nl_filter = nl_filter,
+        # nl_size = nl_size,
+        # sigma = sigma,
+        # dmin = dmin,
+        # dprop = dprop,
+        # hmin = hmin,
+        # crown_prop = crown_prop,
+        # crown_hmin = crown_hmin,
+        .options = options
+      )
+      return(output)
+      #
+      # if LAScluster
+    } else if (lidR::is(las, "LAScluster")) {
+      # retrieve buffer
+      buffer <- las@buffer
+      #
+      x <- lidR::readLAS(las)
+      if (lidR::is.empty(x))
+        return(NULL)
+      # if ROI present
+      if (!is.null(ROI))
+      {
+        # crop ROI to chunk
+        ROI <- sf::st_crop(ROI, las)
+        # crop point cloud to ROI plus buffer
+        x <- lidR::clip_roi(x, sf::st_buffer(ROI, buffer))
+      }
+      if (lidR::is.empty(x))
+        return(NULL)
+      #
+      output <- tree_detection(
+        x,
+        res = res,
+        ROI = ROI,
+        normalize = normalize,
+        crown = crown,
+        ...
+        # nl_filter = nl_filter,
+        # nl_size = nl_size,
+        # sigma = sigma,
+        # dmin = dmin,
+        # dprop = dprop,
+        # hmin = hmin,
+        # crown_prop = crown_prop,
+        # crown_hmin = crown_hmin
+      )
+      if (is.null(output)) return(NULL)
+      if (nrow(output) == 0) return(NULL)
+      # if no ROI
+      if (is.null(ROI))
+      {
+        # crop results to bounding box of chunk
+        sf::st_agr(output) = "constant"
+        output <- sf::st_crop(output, las)
+      }
+      if (nrow(output) == 0) {
+        return(NULL)
+      } else {
+        return(output)
+      }
+      #
+      # if LAS
+    } else {
+      if (lidR::is(las, "LAS")) {
+        #
+        if (normalize)
+        {
+          las <- lidR::normalize_height(las, algorithm = lidR::tin())
+        }
+        # compute chm from points
+        las <-
+          lidR::rasterize_canopy(las, res = res, algorithm = lidR::p2r())
+      } # if raster
+      if (inherits(las, "SpatRaster"))
+      {
+        segms <- lidaRtRee::tree_segmentation(
+          las,
+          ...
+          # nl_filter = nl_filter,
+          # nl_size = nl_size,
+          # sigma = sigma,
+          # dmin = dmin,
+          # dprop = dprop,
+          # hmin = hmin,
+          # crown_prop = crown_prop,
+          # crown_hmin = crown_hmin
+        )
+        # extract apices
+        output <-
+          lidaRtRee::tree_extraction(segms, crown = crown)
+        if (is.null(output)) return(NULL)
+        # if ROI
+        if (!is.null(ROI))
+        {
+          sf::st_agr(output) = "constant"
+          output <- sf::st_intersection(output, ROI)
+        }
+        if (nrow(output) == 0) {
+          return(NULL)
+        }
+        # crowns are now handled directly by tree_extraction
+        # if (crown)
+        # {
+        #   # segments_id outside of ROI to NA
+        #   if (!is.null(ROI))
+        #   {
+        #     segms$segments_id[!is.element(segms$segments_id, output$id)] <- NA
+        #   }
+        # }
+        return(output)
+      }
+      stop("Not supported input")
+    }
+  }
+
 #-------------------------------------------------------------------------------
 #' Disk-shaped matrix mask
 #'
@@ -598,8 +840,9 @@ seg_adjust <- function(dem_w, dem_wh, dem_nl, prop = 0.3, min.value = 2, min.max
 #' segments2$segments_id[segments2$segments_id == 0] <- NA
 #' terra::plot(segments2$segments_id %% 8, main = "Segments2", col = rainbow(8))
 #' @seealso \code{\link{dem_filtering}}, \code{\link{maxima_detection}}, 
-#' \code{\link{maxima_detection}}, \code{\link{maxima_selection}}, 
-#' \code{\link{segmentation}}, \code{\link{seg_adjust}}, \code{\link{tree_extraction}}
+#' \code{\link{maxima_selection}}, \code{\link{segmentation}}, 
+#' \code{\link{seg_adjust}}, \code{\link{tree_extraction}},
+#' \code{\link{tree_detection}}
 #' @return A SpatRaster with 4 layers: selected local maxima (values = 
 #' distance to higher pixel), segments, non-linear preprocessed dem, smoothed 
 #' preprocessed dem
@@ -655,13 +898,22 @@ tree_segmentation <- function(dem, nl_filter = "Closing", nl_size = 5, sigma = 0
 #-------------------------------------------------------------------------------
 #' Tree extraction
 #'
-#' creates a dataframe with segment id, height and coordinates of maxima, surface and volume, computed from three images: initial, local maxima and segmented, obtained with \code{\link{tree_segmentation}}
+#' creates a data.frame with segment id, height and coordinates of maxima, surface and volume, computed from three images: 
+#' initial, local maxima and segmented, obtained with \code{\link{tree_segmentation}}. The 2D polygon associated to each crown 
+#' can be added as a WKT field
 #'
-#' @param r_dem_nl SpatRaster object. raster of canopy height model, preferably filtered to avoid effect of holes on volume and surface computation
-#' @param r_maxi SpatRaster object. raster with positive values at local maxima
-#' @param r_dem_w SpatRaster object. segmented raster
+#' @param r_dem_nl SpatRaster object. Output raster of \code{\link{tree_segmentation}}. Otherwise a raster of canopy height model,
+#' preferably filtered to avoid effect of holes on volume and surface computation can be provided. In this case arguments `r_maxi`, `r_dem_w` 
+#' have to be provided
+#' @param r_maxi SpatRaster object. raster with positive values at local maxima (in case `r_dem_nl` does not contain it)
+#' @param r_dem_w SpatRaster object. segmented raster  (in case `r_dem_nl` does not contain it)
 #' @param r_mask SpatRaster object. only segments which maxima are inside the mask are extracted. Values should be NA outside the mask, 1 inside.
-#' @return A sf collection of POINTs with 7 fields: tree id, local maximum stats (height, dominance radius), segment stats (surface and volume), coordinates (x and y).
+#' @param crown boolean. Should the 2D crown geometry be added in wkt format 
+#' to the output data.frame ?
+#' @return A sf collection of POINTs with 7 fields: tree id, local maximum stats
+#'  (height, dominance radius), segment stats (surface and volume), coordinates 
+#'  (x and y). In case argument `crown` is `TRUE`, a `crown` field 
+#'  containing the WKT geometry of the 2D crown is also present.
 #' @examples
 #' data(chm_chablais3)
 #' chm_chablais3 <- terra::rast(chm_chablais3)
@@ -670,26 +922,40 @@ tree_segmentation <- function(dem, nl_filter = "Closing", nl_size = 5, sigma = 0
 #' segments <- tree_segmentation(chm_chablais3)
 #'
 #' # tree extraction
-#' trees <- tree_extraction(
-#'   segments$filled_dem, segments$local_maxima,
-#'   segments$segments_id
-#' )
-#' trees
+#' trees <- tree_extraction(segments, crown = TRUE)
+#' # create crown polygons from WKT field
+#' trees_crowns <- sf::st_as_sf(sf::st_drop_geometry(trees), wkt = "crown")
+#' 
+#' # summary of trees without wkt field
+#' summary(trees[, -which(names(trees) == "crown")])
 #'
 #' # plot initial image
 #' terra::plot(chm_chablais3)
 #'
 #' # add treetop positions
 #' plot(trees["h"], add = TRUE, cex = trees$h/20, col = "black")
-#' \donttest{
-#' # add segment contours (vectorization is slow)
-#' contours <- terra::as.polygons(segments$segments_id)
-#' terra::plot(contours, add = TRUE, border = "white")
-#' }
+#' # add crowns
+#' plot(sf::st_geometry(trees_crowns), add = TRUE, border = "black", col = NA)
 #'
-#' @seealso \code{\link{tree_segmentation}}
+#' @seealso \code{\link{tree_segmentation}}, \code{\link{tree_detection}}
 #' @export
-tree_extraction <- function(r_dem_nl, r_maxi, r_dem_w, r_mask = NULL) {
+tree_extraction <- function(r_dem_nl, r_maxi = NULL, r_dem_w = NULL, r_mask = NULL, crown = FALSE) {
+  # if first argument is an output of tree_segmentation
+  if(length(names(r_dem_nl)) == 4)
+  {
+    if (all(names(r_dem_nl) == c("local_maxima", "segments_id", "filled_dem", "smoothed_dem")))
+    {
+      r_dem_w <- r_dem_nl$segments_id
+      r_maxi <- r_dem_nl$local_maxima
+      r_dem_nl <- r_dem_nl$filled_dem
+    } else { # check that other rasters are provided if first one is not multi-layer
+      if(is.null(r_maxi) & is.null(r_dem_w))
+      {
+        warning("Arguments r_maxi and r_dem_w should be provided if r_dem_nl is not multi-layer")
+        return(NULL)
+      }
+    }
+  }
   # segments surface
   s <- data.frame(t(table(terra::values(r_dem_w))))[, -1]
   names(s) <- c("id", "s")
@@ -709,7 +975,8 @@ tree_extraction <- function(r_dem_nl, r_maxi, r_dem_w, r_mask = NULL) {
     names(sp) <- c("id", "sp")
     sp$sp <- sp$sp * terra::xres(r_dem_nl) * terra::yres(r_dem_nl)
     # segments volume in mask
-    vp <- stats::aggregate(terra::values(r_dem_nl * r_mask), by = list(terra::values(r_dem_w)), FUN = sum)
+    vp <- stats::aggregate(terra::values(r_dem_nl * r_mask),
+                           by = list(terra::values(r_dem_w)), FUN = sum)
     names(vp) <- c("id", "vp")
     vp$vp <- vp$vp * terra::xres(r_dem_nl) * terra::yres(r_dem_nl)
   }
@@ -720,20 +987,40 @@ tree_extraction <- function(r_dem_nl, r_maxi, r_dem_w, r_mask = NULL) {
   if (length(cells) == 0) {
     segms <- NULL
   } else {
+    # extract coordinates
     coord <- terra::xyFromCell(r_maxi, cells)
+    # create data.frame
     segms <- data.frame(coord, id = terra::values(r_dem_w)[cells], 
                         h = terra::values(r_dem_nl)[cells], 
                         dom_radius = terra::values(r_maxi)[cells])
+    # add surface and volume
     segms <- merge(segms, s, all.x = TRUE)
     segms <- merge(segms, v, all.x = TRUE)
+    # add surface and volume in plot if mask present
     if (!is.null(r_mask)) {
       segms <- merge(segms, sp, all.x = TRUE)
       segms <- merge(segms, vp, all.x = TRUE)
     }
+    # duplicate coordinates 
     segms$X <- segms$x
     segms$Y <- segms$y
-    segms <- sf::st_as_sf(segms, coords = c("X", "Y"))
-    sf::st_crs(segms) <- terra::crs(r_dem_nl)
+    # convert to sf
+    segms <- sf::st_as_sf(segms, coords = c("X", "Y"),
+                          crs = terra::crs(r_dem_nl))
+    # if output crowns
+    if(crown)
+    {
+      vecteur <- terra::as.polygons(r_dem_w)
+      # convert to sf
+      vecteur <- sf::st_as_sf(vecteur)
+      # extract geom to attribute
+      vecteur$crown <- sf::st_as_text(sf::st_geometry(vecteur))
+      # rename first attribute for merging
+      names(vecteur)[1] <- "id"
+      # add polygon geometry attribute to output
+      segms <-
+        merge(segms, sf::st_drop_geometry(vecteur), all.x = TRUE)
+    }
   }
   segms
 }
